@@ -1,4 +1,4 @@
-# Copyright 2018 John Reese
+# Copyright 2022 Amethyst Reese
 # Licensed under the MIT license
 
 """
@@ -26,6 +26,11 @@ from typing import (
 )
 from warnings import warn
 
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
+
 from .context import contextmanager
 from .cursor import Cursor
 
@@ -34,6 +39,9 @@ __all__ = ["connect", "Connection", "Cursor"]
 LOG = logging.getLogger("aiosqlite")
 
 
+IsolationLevel = Optional[Literal["DEFERRED", "IMMEDIATE", "EXCLUSIVE"]]
+
+
 def get_loop(future: asyncio.Future) -> asyncio.AbstractEventLoop:
     if sys.version_info >= (3, 7):
         return future.get_loop()
@@ -68,16 +76,12 @@ class Connection(Thread):
 
         return self._connection
 
-    def _execute_insert(
-        self, sql: str, parameters: Iterable[Any]
-    ) -> Optional[sqlite3.Row]:
+    def _execute_insert(self, sql: str, parameters: Any) -> Optional[sqlite3.Row]:
         cursor = self._conn.execute(sql, parameters)
         cursor.execute("SELECT last_insert_rowid()")
         return cursor.fetchone()
 
-    def _execute_fetchall(
-        self, sql: str, parameters: Iterable[Any]
-    ) -> Iterable[sqlite3.Row]:
+    def _execute_fetchall(self, sql: str, parameters: Any) -> Iterable[sqlite3.Row]:
         cursor = self._conn.execute(sql, parameters)
         return cursor.fetchall()
 
@@ -177,7 +181,9 @@ class Connection(Thread):
             self._connection = None
 
     @contextmanager
-    async def execute(self, sql: str, parameters: Iterable[Any] = None) -> Cursor:
+    async def execute(
+        self, sql: str, parameters: Optional[Iterable[Any]] = None
+    ) -> Cursor:
         """Helper to create a cursor and execute the given query."""
         if parameters is None:
             parameters = []
@@ -186,7 +192,7 @@ class Connection(Thread):
 
     @contextmanager
     async def execute_insert(
-        self, sql: str, parameters: Iterable[Any] = None
+        self, sql: str, parameters: Optional[Iterable[Any]] = None
     ) -> Optional[sqlite3.Row]:
         """Helper to insert and get the last_insert_rowid."""
         if parameters is None:
@@ -195,7 +201,7 @@ class Connection(Thread):
 
     @contextmanager
     async def execute_fetchall(
-        self, sql: str, parameters: Iterable[Any] = None
+        self, sql: str, parameters: Optional[Iterable[Any]] = None
     ) -> Iterable[sqlite3.Row]:
         """Helper to execute a query and return all the data."""
         if parameters is None:
@@ -248,7 +254,8 @@ class Connection(Thread):
                 warnings.warn(
                     "Deterministic function support is only available on "
                     'Python 3.8+. Function "{}" will be registered as '
-                    "non-deterministic as per SQLite defaults.".format(name)
+                    "non-deterministic as per SQLite defaults.".format(name),
+                    stacklevel=2,
                 )
 
             await self._execute(self._conn.create_function, name, num_params, func)
@@ -258,27 +265,27 @@ class Connection(Thread):
         return self._conn.in_transaction
 
     @property
-    def isolation_level(self) -> str:
+    def isolation_level(self) -> Optional[str]:
         return self._conn.isolation_level
 
     @isolation_level.setter
-    def isolation_level(self, value: str) -> None:
+    def isolation_level(self, value: IsolationLevel) -> None:
         self._conn.isolation_level = value
 
     @property
-    def row_factory(self) -> "Optional[Type]":  # py3.5.2 compat (#24)
+    def row_factory(self) -> Optional[Type]:
         return self._conn.row_factory
 
     @row_factory.setter
-    def row_factory(self, factory: "Optional[Type]") -> None:  # py3.5.2 compat (#24)
+    def row_factory(self, factory: Optional[Type]) -> None:
         self._conn.row_factory = factory
 
     @property
-    def text_factory(self) -> Type:
+    def text_factory(self) -> Callable[[bytes], Any]:
         return self._conn.text_factory
 
     @text_factory.setter
-    def text_factory(self, factory: Type) -> None:
+    def text_factory(self, factory: Callable[[bytes], Any]) -> None:
         self._conn.text_factory = factory
 
     @property
@@ -348,16 +355,13 @@ class Connection(Thread):
         pages: int = 0,
         progress: Optional[Callable[[int, int, int], None]] = None,
         name: str = "main",
-        sleep: float = 0.250
+        sleep: float = 0.250,
     ) -> None:
         """
         Make a backup of the current database to the target database.
 
         Takes either a standard sqlite3 or aiosqlite Connection object as the target.
         """
-        if sys.version_info < (3, 7):
-            raise RuntimeError("backup() method is only available on Python 3.7+")
-
         if isinstance(target, Connection):
             target = target._conn
 
@@ -376,7 +380,7 @@ def connect(
     *,
     iter_chunk_size=64,
     loop: Optional[asyncio.AbstractEventLoop] = None,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Connection:
     """Create and return a connection proxy to the sqlite database."""
 
